var Class = require("core").Class;
var BinaryData = require("core").Data.BinaryData;
var Request = require("core").Net.Request;
var Response = require("core").Net.Response;
var RequestProcessor = require("core").Net.RequestProcessor;
var JsonRpcRequest = require('core').JsonRpc.JsonRpcRequest;
var JsonRpcResponse = require('core').JsonRpc.JsonRpcResponse;
var JsonRpcAbstract = require('core').JsonRpc.JsonRpcAbstract;
var ProxyLoader = require('./ProxyLoader.js').ProxyLoader;
var ErrorEvent = require('core').Event.ErrorEvent;

/**
 * @class ProxyRequestProcessor Base class for Request processor family.
 * @param {Request} request
 * @param {Response} response
 */
var ProxyRequestProcessor = function(){
    this.Extends = RequestProcessor;

    this._request = null;
    this._response = null;
    
    this._jsonRpcRequest = null;
    
    /**
     * Main processing method.
     */
    this.process = function(request, response){
        console.log('  -- URI: ' + request.getUrl());
        console.time('   -- REQUEST TIME');
        
        this._request = request;
        this._response = response;
        
        try {
            this._jsonRpcRequest = new JsonRpcRequest(request.getBody().toString());
        } catch(ex) {
            this._onError(new ErrorEvent(JsonRpcAbstract.Exceptions.INPROPER_JSON_STRING, null, -32000, ex));
            return;
        }
        
        try {
            var proxyLoader = new ProxyLoader(
                new Request(this._jsonRpcRequest.getParam('request')),
                this.server.getConfiguration()
            );
                
            proxyLoader.addEventListener(ProxyLoader.LOADED, this._onSuccess, this);
            proxyLoader.addEventListener(ProxyLoader.ERROR,  this._onError,   this);
            proxyLoader.load();
         } catch(e) {
            this._onError(e);
         }
    };

    this._onSuccess = function(e){
        console.log('  -- proxy loader COMPLETE. sending response...');

		var response = e.data;

		//XXX: usunalem content-type
        this._response.setStatusCode(200);
        this._response.setHeader('Access-Control-Allow-Origin', '*');

		//TODO: wszystko zwracamy jako http a nie jako json
		//if(response._isBinary()){
			this._response.setBody(response.toBinaryData());
		/*}else{
			this._response.setHeader('Content-Type', 'application/json; charset=utf-8');

	        var rpc = new JsonRpcResponse();
	        rpc.setId(this._jsonRpcRequest.getId());
	        rpc.setResult(response.toJson());

			this._response.setBody(new BinaryData(rpc.toString(), BinaryData.Encoding.TEXT, BinaryData.CharacterEncoding.UTF8));
		}*/

        this._response.send();

        console.timeEnd('   -- REQUEST TIME');
        console.log('-- request ended');
    };

    this._onError = function(e){
        console.log('  -- proxy loader ERROR. sending error response...');

        if(typeof e ==='string'){
            console.log('     -- ' + e);
        } else {
            console.log('     -- ' + e.code + ': ' + e.message);
        }
       
        var jsonRpcResponse = new JsonRpcResponse();
        if(this._jsonRpcRequest){
            jsonRpcResponse.setId(this._jsonRpcRequest.getId());
        } else {
            jsonRpcResponse.setId('error');
        }
        
        if(typeof e ==='string'){
            jsonRpcResponse.setError(-32000, e, null);
        } else {
            jsonRpcResponse.setError(e.code, e.message, e.data);
        }

        this._setResponseHeaders();

        this._response.setBody(new BinaryData(jsonRpcResponse.toString(), BinaryData.Encoding.TEXT, BinaryData.CharacterEncoding.UTF8));
        this._response.send();
        console.timeEnd('   -- REQUEST TIME');
        console.log('-- request ended');
    };
    
    this._setResponseHeaders = function(){
        this._response.setStatusCode(200);
        this._response.setHeader('Content-Type', 'application/json; charset=utf-8');
        this._response.setHeader('Access-Control-Allow-Origin', '*');
    };
}
ProxyRequestProcessor = new Class( new ProxyRequestProcessor() );
exports.ProxyRequestProcessor = ProxyRequestProcessor; 
