var Class = require("core").Class;
var RequestResponseDecorator = require("./RequestResponseDecorator.js").RequestResponseDecorator;
var AbstractDecorator = require("core").Decorator.AbstractDecorator;
var Event = require('core').Event.Event;
var ErrorEvent = require('core').Event.ErrorEvent;
var Request = require('core').Net.Request;
var Response = require('core').Response;


/**
 * @class CacheControlDecorator Responsible for cache-control header management.
 * @extends RequestResponseDecorator
 */
var CacheControlDecorator = function(){
    this.Extends = RequestResponseDecorator;
    /**
     * Checks and change cache-control header to <i>"server"</i> value, except <i>"no-cache"</i> value wich will remain unchanged. 
     * @public
     * @param {Response} response
     */
    this.decorateResponse = function(response){
        // if response is really Response Object
        if(response instanceof Response){
            var cache = response.getHeader('cache-control');

            if(cache != "server" && cache != "no-cache"){
                response.setHeader('cache-control', "server", true);
            }
            this.dispatchEvent(new Event(AbstractDecorator.COMPLETE, null));
        } else {
            this.dispatchEvent(new ErrorEvent(AbstractDecorator.ERROR, null, -3200, "CacheControlDecorator: Give me Response Object please!"));
        }
    };
};

CacheControlDecorator = new Class( new CacheControlDecorator() );
exports.CacheControlDecorator = CacheControlDecorator;
