var Class = require("core").Class;
var RequestResponseDecorator = require("./RequestResponseDecorator.js").RequestResponseDecorator;
var AbstractDecorator = require("core").Decorator.AbstractDecorator;
var Event = require('core').Event.Event;
var ErrorEvent = require('core').Event.ErrorEvent;
var Request = require('core').Net.Request;
var Response = require('core').Net.Response;


/**
 * @class CookieDecorator Responsible for maintainig cookie policy. Can allow or dissalow Onet's cookie foreward or cookie setting response.
 * @extends RequestResponseDecorator
 * @param {Object} configuration Configuration object. <br/><i>Example configuration object:</i>
 *                 <pre>
 *                  {
 *                      "disallowSet" : true,
 *                      "forwardOnetCookie" : false   
 *                  }
 *                 </pre><br/>
 *                 <dl>
 *                      <dt>disallowSet</dt><dd>Indicates if response can set cookies. Default is <i>false</i> so every response can set cookies</dd>
 *                      <dt>forwardOnetCookie</dt><dd>Indicates if Onet's cookies should be forwarded</dd>
 *                 </dl>
 * @param {Array} contentTypes Array of Strings representing various mime types like: <i>text/html</i>, <i>application/json</i> or patterns <i>application/*</i>
 */
var CookieDecorator = function(){
    this.Extends = RequestResponseDecorator;
    /**
     * Processing request. Depending on configuration removes or not onet's cookies.
     * @public
     * @param {Request} request
     */
    this.decorateRequest = function(request){
        // if request is really Request Object
        if(request instanceof Request){
            if( !(this.configuration.hasOwnProperty("forwardOnetCookie") ) ){

                var cookies = request.getHeader("cookie");
                //FIXME poprawilem bo tu byl blad nei wiem co ma byc
                var pattern = /XXXXX/i;

                for(var cookieName in cookies){
                    if(cookieName.match("onet") != null){
                        request.removeCookie(cookieName);
                    }
                }
            }
            this.dispatchEvent(new Event(AbstractDecorator.COMPLETE, null));
        } else {
            this.dispatchEvent(new ErrorEvent(AbstractDecorator.ERROR, null, -3200, "CookieDecorator: Give me Request Object please!"));
        }
        
    };
    /**
     * Processing response. Depending on configuration removes or not set-cookie header.
     * @public
     * @param {Response} response
     */
    this.decorateResponse = function(response){
        // if response is really Response Object
        if(response instanceof Response){
            if( this.configuration.hasOwnProperty("disallowSet") && this.configuration.disallowSet ){
                response.removeHeader("set-cookie");
            }
            this.dispatchEvent(new Event(AbstractDecorator.COMPLETE, null));
        } else {
            this.dispatchEvent(new ErrorEvent(AbstractDecorator.ERROR, null, -3200, "CookieDecorator: Give me Response Object please!"));
        }
    };   

};
CookieDecorator = new Class( new CookieDecorator() );
exports.CookieDecorator = CookieDecorator;
