var Class = require("core").Class;
var RequestResponseDecorator = require("./RequestResponseDecorator.js").RequestResponseDecorator;
var AbstractDecorator = require("core").Decorator.AbstractDecorator;
var Event = require('core').Event.Event;
var ErrorEvent = require('core').Event.ErrorEvent;
var Request = require('core').Net.Request;
var Response = require('core').Net.Response;


/**
 * @class HeaderDecorator Responsible for cache-control header management.
 * @extends RequestResponseDecorator
 */
var HeaderDecorator = function(){
    this.Extends = RequestResponseDecorator;
    
    
    this.decorateRequest = function(request){
        // if response is really Response Object
        
        if(request instanceof Request){
            
            if(this.configuration.hasOwnProperty('request')){
                for(var i=0,max=this.configuration.request.length;i<max;i++){
                    request.setHeader(
                        this.configuration.request[i].header, 
                        this.configuration.request[i].value, 
                        this.configuration.request[i].override
                    );
                }
            }           
            
            this.dispatchEvent(new Event(AbstractDecorator.COMPLETE, null));
        } else {
            this.dispatchEvent(new ErrorEvent(AbstractDecorator.ERROR, null, -3200, "HeaderDecorator: Give me Response Object please!"));
        }
    };
    
    
    
    /**
     * Checks and change cache-control header to <i>"server"</i> value, except <i>"no-cache"</i> value wich will remain unchanged. 
     * @public
     * @param {Response} response
     */
    this.decorateResponse = function(response){
        // if response is really Response Object
        if(response instanceof Response){

            if(this.configuration.hasOwnProperty('response')){
                for(var i=0,max=this.configuration.response.length;i<max;i++){
                    response.setHeader(
                        this.configuration.response[i].header, 
                        this.configuration.response[i].value, 
                        this.configuration.response[i].override
                    );
                }
            }           
            
            this.dispatchEvent(new Event(AbstractDecorator.COMPLETE, null));
        } else {
            this.dispatchEvent(new ErrorEvent(AbstractDecorator.ERROR, null, -3200, "HeaderDecorator: Give me Response Object please!"));
        }
    };
};

HeaderDecorator = new Class( new HeaderDecorator() );
exports.HeaderDecorator = HeaderDecorator;
