var Class = require("core").Class;
var Loader = require('core').Net.Loader;
var RequestResponseDecorator = require("./RequestResponseDecorator.js").RequestResponseDecorator;
var AbstractDecorator = require("core").Decorator.AbstractDecorator;
var Request = require('core').Net.Request;
var Event = require('core').Event.Event;
var ErrorEvent = require('core').Event.ErrorEvent;


/**
 * @class RevProxyLayoutDecorator
 * @extends RequestResponseDecorator
 */
var RevProxyLayoutDecorator = function(){
    this.Extends = RequestResponseDecorator;
    this._request = null;
    this._response = null;
    this.decorateRequest = function(request){
        this._request = request;
        this.dispatchEvent(new Event(AbstractDecorator.COMPLETE, null));
    };
    this.decorateResponse = function(response){
        this._response = response;

        if(this.configuration.hasOwnProperty("url")){
            var loaderRequest = new Request(this.configuration.url);
           
            //TODO clone headers - and prepare for opal call
            var loader = new Loader(loaderRequest);
            loader.addEventListener(Loader.Event.LOADED, this._onLoad, this);
            loader.addEventListener(Loader.Event.ERROR, this._onError, this);
            loader.load();
        } else {
            this.dispatchEvent(new ErrorEvent(AbstractDecorator.ERROR, null, -32000, "There is no URL param for RevProxyLayoutDecorator"));
        }
    };

    this._onLoad = function(e){

        if(e.data.getStatusCode()!==200){
            this.dispatchEvent(new ErrorEvent(AbstractDecorator.ERROR, null, -32000, "Layout has wrong status code: " + e.data.getStatusCode()));
            return;
        }

        var skeletonBody = e.data.getBody();
        var body = this._response.getBody();

        // replacing revproxy:title to <cdf:target id="title""
        var matches = skeletonBody.match(/<!--\s+revproxy:([a-zA-Z0-9_]+)\s+-->/gim);

        // if there are matches then replace them
        if(matches){
            for(var i=0, max=matches.length;i<max;i++){
                var revProxyTarget = matches[i].match(/<!--\s+revproxy:([a-zA-Z0-9_]+)\s+-->/i);
                if(revProxyTarget.length===2){
                    var cdfTarget = '<cdf:target id="' + revProxyTarget[1] + '"/>\r\n';
                    skeletonBody = skeletonBody.replace(revProxyTarget[0], cdfTarget);
                }
            }
        }
        // wrapping with skeleton tag
        body = body + '<cdf:model target=".skeleton" content-type="text/html">\r\n' + skeletonBody + "</cdf:model>";

        // setting new body
        this._response.setBody(body);

        // i'm all done
        this.dispatchEvent(new Event(AbstractDecorator.COMPLETE, null));
    };
    
    this._onError = function(e){
        this.dispatchEvent(new Event(AbstractDecorator.ERROR, e, e.code, e.message));
    };

};

RevProxyLayoutDecorator = new Class( new RevProxyLayoutDecorator() );
exports.RevProxyLayoutDecorator = RevProxyLayoutDecorator;
