var Class = require("core").Class;
var RequestResponseDecorator = require("./RequestResponseDecorator.js").RequestResponseDecorator;
var AbstractDecorator = require("core").Decorator.AbstractDecorator;
var Event = require('core').Event.Event;
var ErrorEvent = require('core').Event.ErrorEvent;
var Request = require('core').Net.Request;
var Response = require('core').Net.Response;
var BinaryData = require('core').Data.BinaryData;


/**
 * @class StaticStringDecorator Responsible for cache-control header management.
 * @extends RequestResponseDecorator
 */
var StaticStringDecorator = function(){
    this.Extends = RequestResponseDecorator;
    
    this._decorate = false;
    
    
    this.decorateRequest = function(request){
        try{
            if(request.getUrl()==='http://www.onet.pl:80/'){
                this._decorate = true;
            }
        }catch(e){
            console.log(e);
        }
        
        this.dispatchEvent(new Event(AbstractDecorator.COMPLETE, null));
    };
    
    /**
     * Checks and change cache-control header to <i>"server"</i> value, except <i>"no-cache"</i> value wich will remain unchanged. 
     * @public
     * @param {Response} response
     */
    this.decorateResponse = function(response){
        // if response is really Response Object
        
        if(response instanceof Response){
            
            if(this._decorate){
            
                try {
                    var orginalBd = response.getBody();

                    var content = orginalBd.toUTF8String().replace(
                        "</body>", 
                        this.configuration.value + "</body>"
                    );
                    orginalBd.setData(content, BinaryData.Encoding.TEXT, BinaryData.CharacterEncoding.UTF8);

                } catch(ex){
                    console.log(ex);
                }
            }
            
            
            this.dispatchEvent(new Event(AbstractDecorator.COMPLETE, null));
        } else {
            this.dispatchEvent(new ErrorEvent(AbstractDecorator.ERROR, null, -3200, "StaticStringDecorator: Give me Response Object please!"));
        }
    };
};

StaticStringDecorator = new Class( new StaticStringDecorator() );
exports.StaticStringDecorator = StaticStringDecorator;
