describe('DecoratorChain', function(){
    
    var AbstractDecorator = null;
    var DecoratorChain = null;
    var ExampleDecoratorClass = null;
    
    it("require", function(){
        AbstractDecorator = require('../../../lib/core/decorator/AbstractDecorator.js').AbstractDecorator;
        DecoratorChain = require('../../../lib/core/decorator/DecoratorChain.js').DecoratorChain;
        Event = require('../../../lib/core/event/Event.js').Event;
        Class = require('../../../lib/mootools/mootools-node.js').Class;
        
        ExampleDecoratorClass = function(){
            this.Extends = AbstractDecorator;
            this.initialize = function(){    
            };
            this.decorate = function(obj){
                obj.example = "test";
                var that = this;
                setTimeout(function(){
                    that.dispatchEvent(new Event(AbstractDecorator.COMPLETE));
                }, 100);
            };
        };
        ExampleDecoratorClass = new Class( new ExampleDecoratorClass() );
        
        Example2DecoratorClass = function(){
            this.Extends = AbstractDecorator;
            this.initialize = function(){    
            };
            this.decorate = function(obj){
                obj.example = "test2";
                this.dispatchEvent(new Event(AbstractDecorator.COMPLETE));
            };
        };
        Example2DecoratorClass = new Class( new Example2DecoratorClass() );
    });
    
    it("creation", function(){
        var chain = new DecoratorChain();
        expect(chain.currentPhase).toEqual(DecoratorChain.STOPPED);
    });
    
    it("addDecorator", function(){ 
        var chain = new DecoratorChain();
        var decorator = new ExampleDecoratorClass();
        expect( decorator.hasEventListener(AbstractDecorator.COMPLETE) ).toBeFalsy();
        expect( chain.hasDecorator(decorator) ).toBeFalsy();
        chain.addDecorator(decorator);
        expect( decorator.hasEventListener(AbstractDecorator.COMPLETE) ).toBeTruthy();
        expect( chain.hasDecorator(decorator) ).toBeTruthy();
    });
    it("addDecorator Exeption", function(){ 
        var chain = new DecoratorChain();

        expect(function(){
            chain.addDecorator({});
        }).toThrow(DecoratorChain.Exeption.WRONG_DECORATOR_TYPE);

    });
    it("removeDecorator", function(){
        var chain = new DecoratorChain();
        var decorator = new ExampleDecoratorClass();
        
        chain.addDecorator(decorator);
        
        expect(decorator.hasEventListener(AbstractDecorator.COMPLETE)).toBeTruthy();
        expect(chain.hasDecorator(decorator)).toBeTruthy();
        
        chain.removeDecorator(decorator);
        
        expect(decorator.hasEventListener(AbstractDecorator.COMPLETE)).toBeFalsy();
        expect(chain.hasDecorator(decorator)).toBeFalsy();
    });
    it("removeDecorator Exeption", function(){ 
        var chain = new DecoratorChain();
        var decorator = new ExampleDecoratorClass();
        chain.addDecorator(decorator);
        expect(function(){
            chain.removeDecorator({});
        }).toThrow(DecoratorChain.Exeption.WRONG_DECORATOR_TYPE);

    });
    it("decoration process", function(){
        runs(function(){
            var chain = new DecoratorChain();
            var decorator = new ExampleDecoratorClass();
            var decorator2 = new Example2DecoratorClass();
            this.complete = false;
            
            this.callbackFuns = function(e){
                this.complete = true;
            };
            
            chain.addDecorator(decorator);
            chain.addDecorator(decorator2);
            chain.addEventListener(DecoratorChain.Event.COMPLETE, this.callbackFuns, this);
            
            this.obj = { example : "start" };
            
            chain.decorate(this.obj, 'decorate');    
        });
        waits(200);
        runs(function(){
           expect(this.obj.example).toEqual("test2"); 
           expect(this.complete).toBeTruthy();
        });
    });
    it("decoration wrong params exeption", function(){
        var chain = new DecoratorChain();
        expect(function(){
            chain.decorate();
        }).toThrow(DecoratorChain.Exeption.WRONG_PARAMS);
    });
    it("decoration wrong params exeption", function(){
        var chain = new DecoratorChain();
        var decorator = new ExampleDecoratorClass();
        var decorator2 = new Example2DecoratorClass();
        
        chain.addDecorator(decorator);
        chain.addDecorator(decorator2);
        
        this.obj = { example : "start" };
            
        chain.decorate(this.obj, 'decorate');
        
        expect(function(){
            chain.decorate(this.obj, 'decorate');
        }).toThrow(DecoratorChain.Exeption.ALREADY_IN_PROGRESS);
    });
});