describe('JsonRpcError', function(){


    var JsonRpcError = null;
    var jsonRpcErr = null;

    it('can load needed classes', function(){
        expect(function(){
            JsonRpcError = require('../../../lib/core/jsonrpc/JsonRpcError.js').JsonRpcError;
        }).not.toThrow();

    });

    it('proper init/construct', function(){
        expect(function(){
            jsonRpcErr = new JsonRpcError();
        }).not.toThrow();

        expect(function(){
            jsonRpcErr = new JsonRpcError(-3200, 'message');
        }).not.toThrow();

        expect(function(){
            jsonRpcErr = new JsonRpcError(-3201, 'message', {data:'data'});
        }).not.toThrow();

        
    });

    it('not proper init/construct', function(){
        expect(function(){
            jsonRpcErr = new JsonRpcError({data:'data'});
        }).toThrow('Invalid JSON-RPC given');

        expect(function(){
            jsonRpcErr = new JsonRpcError(-3200, {data:'data'});
        }).toThrow('Invalid JSON-RPC given');

        expect(function(){
            jsonRpcErr = new JsonRpcError(-3200, {data:'data'});
        }).toThrow('Invalid JSON-RPC given');

        expect(function(){
            jsonRpcErr = new JsonRpcError('body:[');
        }).toThrow('Inproper JSON string given');

        expect(function(){
            jsonRpcErr = new JsonRpcError(null);
        }).toThrow('Invalid JSON-RPC given');

        expect(function(){
            jsonRpcErr = new JsonRpcError(undefined);
        }).toThrow('Invalid JSON-RPC given');

    });

    it('setters test', function(){

        jsonRpcErr = new JsonRpcError();

        expect(function(){jsonRpcErr.setCode()}).toThrow('Code must be a Number!');
        expect(function(){jsonRpcErr.setCode("code");}).toThrow('Code must be a Number!');
        expect(function(){jsonRpcErr.setCode({})}).toThrow('Code must be a Number!');
        expect(function(){jsonRpcErr.setCode([])}).toThrow('Code must be a Number!');
        expect(function(){jsonRpcErr.setCode(null)}).toThrow('Code must be a Number!');
        expect(function(){jsonRpcErr.setCode(undefined)}).toThrow('Code must be a Number!');
        expect(function(){jsonRpcErr.setCode(-3200)}).not.toThrow();

        expect(function(){jsonRpcErr.setMessage()}).toThrow('Message must be a String!');
        expect(function(){jsonRpcErr.setMessage({})}).toThrow('Message must be a String!');
        expect(function(){jsonRpcErr.setMessage([])}).toThrow('Message must be a String!');
        expect(function(){jsonRpcErr.setMessage(null)}).toThrow('Message must be a String!');
        expect(function(){jsonRpcErr.setMessage(undefined)}).toThrow('Message must be a String!');
        expect(function(){jsonRpcErr.setMessage('message')}).not.toThrow();

        expect(function(){jsonRpcErr.setData(['foo',2])}).not.toThrow();
        expect(function(){jsonRpcErr.setData(new JsonRpcError())}).not.toThrow();
        
    });

    it('getters test', function(){
        var json = {code : -3201, message : 'message', data : {data : 'data'}};
        var data = {data:'data'};
        jsonRpcErr = new JsonRpcError(-3201, 'message', data);
        expect(jsonRpcErr.getCode()).toEqual(-3201);
        expect(jsonRpcErr.getMessage()).toEqual('message');
        expect(jsonRpcErr.getData()).toEqual(data);
        expect(jsonRpcErr.getData()).toBe(data);
        expect(jsonRpcErr.toJson()).toEqual(json);
    });

    it('factory test', function(){
        var data = {data:'data'};
        jsonRpcErr = new JsonRpcError(-3201, 'message');
        expect( JsonRpcError.factory(jsonRpcErr)    ).toEqual(jsonRpcErr);
        jsonRpcErr = new JsonRpcError(-3201, 'message', data);
        expect( JsonRpcError.factory(jsonRpcErr)    ).not.toEqual(jsonRpcErr);
        
        expect(function(){ JsonRpcError.factory(new String("X")) }   ).toThrow("Given something is not a instance of JsonRpcError class");


    });


});