describe('JsonRpcRequest', function(){


    var JsonRpcRequest = null;
    var jsonRpcReq = null;

    it('can load needed classes', function(){
        expect(function(){
            JsonRpcRequest = require('../../../lib/core/jsonrpc/JsonRpcRequest.js').JsonRpcRequest;
        }).not.toThrow();

    });

    it('proper init/construct', function(){
        expect(function(){
            jsonRpcReq = new JsonRpcRequest();
        }).not.toThrow();
    });

    it('proper init/construct', function(){
        expect(function(){
            jsonRpcReq = new JsonRpcRequest('id', 'method', {'config':[]});
        }).not.toThrow();
    });

    it('getters test', function(){
        var params = {'config':[]};
        jsonRpcReq = new JsonRpcRequest('id', 'method', params);
        expect(jsonRpcReq.getId()).toBe('id');
        expect(jsonRpcReq.getMethod()).toBe('method');
        expect(jsonRpcReq.getParams()).toBe(params);
        expect(jsonRpcReq.getVersion()).toBe('2.0');
        expect(jsonRpcReq.getParam('config')).toBe(params.config);
        expect(jsonRpcReq.getParam('config')).toEqual(params.config);
        jsonRpcReq.setParam('foo', 'bar');
        expect(jsonRpcReq.getParam('foo')).toBe('bar');
        expect(jsonRpcReq.getParam('notexist')).toBeNull();

        var json = {jsonrpc: '2.0',
              id: 'id',
              method: 'method',
              params: {config: [], foo: 'bar'} 
        }
        expect(jsonRpcReq.toJson()).toEqual(json);

        var str = '{"jsonrpc":"2.0","id":"id","method":"method","params":{"config":[],"foo":"bar"}}';
        expect(jsonRpcReq.toString()).toEqual(str);
    });

    it('setters test', function(){
        var params = {'config':[]};
        jsonRpcReq = new JsonRpcRequest('id', 'method', params);

        expect(function(){
            jsonRpcReq.setMethod(20);
        }).toThrow('Method should be a string');

        expect(function(){
            jsonRpcReq.setMethod({});
        }).toThrow('Method should be a string');

        expect(function(){
            jsonRpcReq.setMethod([]);
        }).toThrow('Method should be a string');

        expect(function(){
            jsonRpcReq.setMethod(null);
        }).toThrow('Method should be a string');

        expect(function(){
            jsonRpcReq.setMethod(undefined);
        }).toThrow('Method should be a string');
        


        expect(function(){
            jsonRpcReq.setId(20);
        }).toThrow('Id should be a string');

        expect(function(){
            jsonRpcReq.setId({});
        }).toThrow('Id should be a string');

        expect(function(){
            jsonRpcReq.setId([]);
        }).toThrow('Id should be a string');

        expect(function(){
            jsonRpcReq.setId(null);
        }).toThrow('Id should be a string');

        expect(function(){
            jsonRpcReq.setId(undefined);
        }).toThrow('Id should be a string');

        expect(function(){
            jsonRpcReq.setId('3.0');
        }).not.toThrow();



        expect(function(){
            jsonRpcReq.setParam(20, null);
        }).toThrow('Key should be a string');

        expect(function(){
            jsonRpcReq.setParam({}, null);
        }).toThrow('Key should be a string');

        expect(function(){
            jsonRpcReq.setParam([], null);
        }).toThrow('Key should be a string');

        expect(function(){
            jsonRpcReq.setParam(null, null);
        }).toThrow('Key should be a string');

        expect(function(){
            jsonRpcReq.setParam(undefined, null);
        }).toThrow('Key should be a string');

        expect(function(){
            jsonRpcReq.setParam('test', 'test');
        }).not.toThrow();
        
    });


    it('version check', function(){
        expect(function(){
            jsonRpcReq = new JsonRpcRequest('id', 'method', {'config':[]});
        }).not.toThrow();
        
        jsonRpcReq.setVersion('3.0');
        expect(jsonRpcReq.getVersion()).toBe('3.0');


        expect(function(){
            jsonRpcReq.setVersion(20);
        }).toThrow('Version should be a string');

        expect(function(){
            jsonRpcReq.setVersion({});
        }).toThrow('Version should be a string');

        expect(function(){
            jsonRpcReq.setVersion([]);
        }).toThrow('Version should be a string');

    });

    it('not proper init/construct', function(){
        expect(function(){
            jsonRpcReq = new JsonRpcRequest('{error in hiere:}');
        }).toThrow('Inproper JSON string given');
    });

    it('not proper init/construct', function(){
        expect(function(){
            jsonRpcReq = new JsonRpcRequest('{"foo":"bar"}');
        }).toThrow('Invalid JSON-RPC given');
    });

    it('not proper init/construct', function(){
        expect(function(){
            jsonRpcReq = new JsonRpcRequest(false);
        }).toThrow('Invalid JSON-RPC given');
    });

    it('not proper init/construct', function(){
        expect(function(){
            jsonRpcReq = new JsonRpcRequest('id');
        }).toThrow('Inproper JSON string given');
    });

    it('not proper init/construct', function(){
        expect(function(){
            jsonRpcReq = new JsonRpcRequest('id', null);
        }).toThrow('Invalid JSON-RPC given');
    });

});