describe('JsonRpcResponse', function(){


    var JsonRpcResponse = null;
    var JsonRpcError = null;
    var jsonRpcRes = null;

    it('can load needed classes', function(){
        expect(function(){
            JsonRpcResponse = require('../../../lib/core/jsonrpc/JsonRpcResponse.js').JsonRpcResponse;
        }).not.toThrow();

        expect(function(){
            JsonRpcError = require('../../../lib/core/jsonrpc/JsonRpcError.js').JsonRpcError;
        }).not.toThrow();
    });

    it('proper init/construct', function(){
        
        expect(function(){
            jsonRpcRes = new JsonRpcResponse();
        }).not.toThrow();

        expect(function(){
            jsonRpcRes = new JsonRpcResponse('id', {data:5});
        }).not.toThrow();

        expect(function(){
            jsonRpcRes = new JsonRpcResponse( 'id',  {data:5},  {code:5, message:'OK', data: {}} );
        }).not.toThrow();

    });

    it('not proper init/construct', function(){
        expect(function(){
            jsonRpcRes = new JsonRpcResponse({data:'data'});
        }).toThrow('Invalid JSON-RPC given');

        
        expect(function(){
            jsonRpcRes = new JsonRpcResponse(-3200, {data:'data'});
        }).toThrow('Id should be a String');
        
        expect(function(){
            jsonRpcRes = new JsonRpcResponse(-3200, {data:'data'});
        }).toThrow('Id should be a String');
       
        expect(function(){
            jsonRpcRes = new JsonRpcResponse('body:[');
        }).toThrow('Inproper JSON string given');

        expect(function(){
            jsonRpcRes = new JsonRpcResponse(null);
        }).toThrow('Invalid JSON-RPC given');

        expect(function(){
            jsonRpcRes = new JsonRpcResponse(undefined);
        }).toThrow('Invalid JSON-RPC given');

        expect(function(){
            jsonRpcRes = new JsonRpcResponse('id', undefined);
        }).toThrow('Invalid JSON-RPC given');
        
        
    });
    
    
    it('getters test', function(){
        var json = {jsonrpc: '2.0',  id: 'example',  result: {config: 'data'},  error: {code: 5, message: 'OK', data: {}}};
        var data = {config:'data'};
        var error = {code:5, message:'OK', data: {}};
        
        jsonRpcRes = new JsonRpcResponse('example', data, error);
        expect(jsonRpcRes.getId ()).toEqual('example');
        expect(jsonRpcRes.getResult ()).toEqual(data);
        expect(jsonRpcRes.getError() instanceof JsonRpcError).toBeTruthy();
        expect(jsonRpcRes.toJson()).toEqual(json)

    });



    it('setters test', function(){
        
        jsonRpcRes = new JsonRpcResponse();
        jsonRpcRes.setError(5, 'OK', {});
        expect( jsonRpcRes.getError() instanceof JsonRpcError).toBeTruthy();

        var data = {config:'data'};
        var error = {code:5, message:'OK', data: {}};
        jsonRpcRes = new JsonRpcResponse('example', data, error);
        jsonRpcRes.setError(new JsonRpcError(5, 'OK', {}));
        expect( jsonRpcRes.getError() instanceof JsonRpcError).toBeTruthy();

        


    });

    

    


});