describe('Message', function(){
	var Message = null, configuration = {
		"body": "test123123",
		"httpVersion": '1.0',
		"headers": {
			"Host": "test.pl",
			"X-Test": 123
		}
	}, convertedHeaders = {
		"host": "test.pl",
		"x-test": [123]
	};

	it('deps', function(){
		expect(function(){
			Message = require('../../../lib/core/net/Message.js').Message;
		}).not.toThrow();
	});

	it('constructor - arguments', function(){
		expect(function(){ new Message(); }).not.toThrow();
		expect(function(){ new Message(null); }).not.toThrow();
		expect(function(){ new Message(undefined); }).not.toThrow();
		expect(function(){ new Message(123); }).not.toThrow();
		expect(function(){ new Message("123"); }).not.toThrow();
		expect(function(){ new Message([]); }).not.toThrow();
		expect(function(){ new Message({}); }).not.toThrow();
		expect(function(){ new Message(function(){}); }).not.toThrow();
		expect(function(){ new Message(new (function(){})); }).not.toThrow();
	});

	it('constructor - configuration', function(){
		var msg;

		expect(function(){ msg = new Message(configuration); }).not.toThrow();
		expect(msg.getBody()).toEqual(configuration.body);
		expect(msg.getHttpVersion()).toEqual(configuration.httpVersion);

		expect(msg.getHeaders()).toEqual(convertedHeaders);
	});

	it('setting/getting/removing headers - throwing proper exceptions', function(){
		var msg;
		
		expect(function(){ msg = new Message(); }).not.toThrow();

		expect(function(){ msg.setHeaders(); }).toThrow(Message.Exception.HEADERS_MUST_BE_AN_OBJECT);
		expect(function(){ msg.setHeaders(null); }).toThrow(Message.Exception.HEADERS_MUST_BE_AN_OBJECT);
		expect(function(){ msg.setHeaders(undefined); }).toThrow(Message.Exception.HEADERS_MUST_BE_AN_OBJECT);
		expect(function(){ msg.setHeaders(123); }).toThrow(Message.Exception.HEADERS_MUST_BE_AN_OBJECT);
		expect(function(){ msg.setHeaders("123"); }).toThrow(Message.Exception.HEADERS_MUST_BE_AN_OBJECT);
		expect(function(){ msg.setHeaders([]); }).toThrow(Message.Exception.HEADERS_MUST_BE_AN_OBJECT);
		expect(function(){ msg.setHeaders({}); }).not.toThrow(Message.Exception.HEADERS_MUST_BE_AN_OBJECT);
		expect(function(){ msg.setHeaders(function(){}); }).toThrow(Message.Exception.HEADERS_MUST_BE_AN_OBJECT);
		expect(function(){ msg.setHeaders(new (function(){})); }).toThrow(Message.Exception.HEADERS_MUST_BE_AN_OBJECT);
		
		expect(function(){ msg.setHeader(); }).toThrow(Message.Exception.HEADER_NAME_MUST_BE_A_STRING);
		expect(function(){ msg.setHeader(null); }).toThrow(Message.Exception.HEADER_NAME_MUST_BE_A_STRING);
		expect(function(){ msg.setHeader(undefined); }).toThrow(Message.Exception.HEADER_NAME_MUST_BE_A_STRING);
		expect(function(){ msg.setHeader(123); }).toThrow(Message.Exception.HEADER_NAME_MUST_BE_A_STRING);
		expect(function(){ msg.setHeader("123"); }).not.toThrow(Message.Exception.HEADER_NAME_MUST_BE_A_STRING);
		expect(function(){ msg.setHeader("123"); }).toThrow(Message.Exception.WRONG_HEADER_VALUE);
		expect(function(){ msg.setHeader([]); }).toThrow(Message.Exception.HEADER_NAME_MUST_BE_A_STRING);
		expect(function(){ msg.setHeader({}); }).toThrow(Message.Exception.HEADER_NAME_MUST_BE_A_STRING);
		expect(function(){ msg.setHeader(function(){}); }).toThrow(Message.Exception.HEADER_NAME_MUST_BE_A_STRING);
		expect(function(){ msg.setHeader(new (function(){})); }).toThrow(Message.Exception.HEADER_NAME_MUST_BE_A_STRING);

		expect(function(){ msg.setHeader("name"); }).toThrow(Message.Exception.WRONG_HEADER_VALUE);
		expect(function(){ msg.setHeader("name", null); }).toThrow(Message.Exception.WRONG_HEADER_VALUE);
		expect(function(){ msg.setHeader("name", undefined); }).toThrow(Message.Exception.WRONG_HEADER_VALUE);
		expect(function(){ msg.setHeader("name", 123); }).not.toThrow(Message.Exception.WRONG_HEADER_VALUE);
		expect(function(){ msg.setHeader("name", "123"); }).not.toThrow(Message.Exception.WRONG_HEADER_VALUE);
		expect(function(){ msg.setHeader("name", []); }).not.toThrow(Message.Exception.WRONG_HEADER_VALUE);
		expect(function(){ msg.setHeader("name", {}); }).toThrow(Message.Exception.WRONG_HEADER_VALUE);
		expect(function(){ msg.setHeader("name", function(){}); }).toThrow(Message.Exception.WRONG_HEADER_VALUE);
		expect(function(){ msg.setHeader("name", new (function(){})); }).toThrow(Message.Exception.WRONG_HEADER_VALUE);

		expect(function(){ msg.removeHeader(); }).toThrow(Message.Exception.HEADER_NAME_MUST_BE_A_STRING);
		expect(function(){ msg.removeHeader(null); }).toThrow(Message.Exception.HEADER_NAME_MUST_BE_A_STRING);
		expect(function(){ msg.removeHeader(undefined); }).toThrow(Message.Exception.HEADER_NAME_MUST_BE_A_STRING);
		expect(function(){ msg.removeHeader(123); }).toThrow(Message.Exception.HEADER_NAME_MUST_BE_A_STRING);
		expect(function(){ msg.removeHeader("123"); }).not.toThrow(Message.Exception.HEADER_NAME_MUST_BE_A_STRING);
		expect(function(){ msg.removeHeader([]); }).toThrow(Message.Exception.HEADER_NAME_MUST_BE_A_STRING);
		expect(function(){ msg.removeHeader({}); }).toThrow(Message.Exception.HEADER_NAME_MUST_BE_A_STRING);
		expect(function(){ msg.removeHeader(function(){}); }).toThrow(Message.Exception.HEADER_NAME_MUST_BE_A_STRING);
		expect(function(){ msg.removeHeader(new (function(){})); }).toThrow(Message.Exception.HEADER_NAME_MUST_BE_A_STRING);
	});

	it('set header/headers returning this', function(){
		var msg;

		expect(function(){ msg = new Message(); }).not.toThrow();

		expect(msg.setHeaders({})).toBe(msg);
		expect(msg.setHeader('a', 'b')).toBe(msg);
	});

	it('setting/getting headers', function(){
		var msg, headers = {
                'X-Onet-App': 123,
                'X-Onet-Name': "str",
                'X-Onet-Config': [1,2,3,4]
            }, parsedHeaders = {
                'x-onet-app': [123],
                'x-onet-name': ["str"],
                'x-onet-config': [1,2,3,4]
            };

		expect(function(){ msg = new Message(); }).not.toThrow();

		expect(function(){ msg.setHeaders(headers); }).not.toThrow();
		expect(msg.getHeaders()).toEqual(parsedHeaders);

		expect(function(){ msg.setHeader('X-X-X', 555); }).not.toThrow();
		expect(msg.getHeader('X-X-X')).toEqual(555);

		expect(function(){ msg.setHeader('X-X-X', 666); }).not.toThrow();
		expect(msg.getHeader('X-x-x')).toEqual([555,666]);

		expect(function(){ msg.setHeader('x-X-x', 777, true); }).not.toThrow();
		expect(msg.getHeader('x-x-x')).toEqual(777);

		expect(function(){ msg.setHeader('YYY', [1,2]); }).not.toThrow();
		expect(msg.getHeader('yyy')).toEqual([1,2]);

		expect(function(){ msg.setHeader('yYy', [3,4]); }).not.toThrow();
		expect(msg.getHeader('YyY')).toEqual([1,2,3,4]);

		expect(function(){ msg.setHeader('YYy', [5,6,7,8], true); }).not.toThrow();
		expect(msg.getHeader('yyY')).toEqual([5,6,7,8]);

		expect(msg.getHeader('qwe123qwe123')).toBeNull();
	});

	it('speficic headers', function(){
		var msg, headers = {
			'HoSt': 'hoscik',
			'Cookie': 'ciastka',
			'set-COOKIE': 'ohohohoh'
		};

		expect(function(){ msg = new Message(); }).not.toThrow();

		expect(function(){ msg.setHeaders(headers); }).not.toThrow();
		expect(msg.getHeader('host')).toEqual(headers.HoSt);
		expect(msg.getHeader('cookie')).toEqual(headers.Cookie);
		expect(msg.getHeader('set-CooKiE')).toEqual(headers['set-COOKIE']);

		expect(function(){ msg.setHeader('HOST', 'aaa'); }).not.toThrow();
		expect(function(){ msg.setHeader('COOKIE', 'bbb'); }).not.toThrow();
		expect(function(){ msg.setHeader('SET-COOKIE', 'ccc'); }).not.toThrow();

		expect(msg.getHeader('host')).toEqual('aaa');
		expect(msg.getHeader('cookie')).toEqual('bbb');
		expect(msg.getHeader('set-cookie')).toEqual('ccc');
	});

	it('removing headers', function(){
		var msg, headers = {
			'yyy': 123,
			'XXX': 456,
			'QqQ': 789
		}, removedHeaders = {
			'yyy': [123]
		};

		expect(function(){ msg = new Message(); }).not.toThrow();
		expect(function(){ msg.setHeaders(headers); }).not.toThrow();

		expect(function(){ msg.removeHeader('xxx'); }).not.toThrow();
		expect(msg.removeHeader('qQq')).toBeTruthy();
		expect(msg.removeHeader('Eee')).toBeFalsy();

		expect(msg.getHeader('xXx')).toBeNull();
		expect(msg.getHeaders()).toEqual(removedHeaders);
	});

	it('body', function(){
		//FIXME: dodac sprawdzanie typow body

		var msg;

		expect(function(){ msg = new Message(); }).not.toThrow();

		expect(function(){ msg.setBody('aaaa'); }).not.toThrow();
		expect(msg.setBody('bbbb')).toBe(msg);

		expect(msg.getBody()).toEqual('bbbb');
	});

	it('httpVersion', function(){
		var msg;

		expect(function(){ msg = new Message(); }).not.toThrow();

		expect(function(){ msg.setHttpVersion(); }).toThrow(Message.Exception.WRONG_HTTP_VERSION_VALUE);
		expect(function(){ msg.setHttpVersion(null); }).toThrow(Message.Exception.WRONG_HTTP_VERSION_VALUE);
		expect(function(){ msg.setHttpVersion(undefined); }).toThrow(Message.Exception.WRONG_HTTP_VERSION_VALUE);
		expect(function(){ msg.setHttpVersion([]); }).toThrow(Message.Exception.WRONG_HTTP_VERSION_VALUE);
		expect(function(){ msg.setHttpVersion({}); }).toThrow(Message.Exception.WRONG_HTTP_VERSION_VALUE);
		expect(function(){ msg.setHttpVersion(function(){}); }).toThrow(Message.Exception.WRONG_HTTP_VERSION_VALUE);
		expect(function(){ msg.setHttpVersion(new (function(){})); }).toThrow(Message.Exception.WRONG_HTTP_VERSION_VALUE);

		expect(function(){ msg.setHttpVersion(1.1); }).not.toThrow(Message.Exception.WRONG_HTTP_VERSION_VALUE);
		expect(msg.getHttpVersion()).toEqual(1.1);

		expect(function(){ msg.setHttpVersion("1.0"); }).not.toThrow(Message.Exception.WRONG_HTTP_VERSION_VALUE);
		expect(msg.getHttpVersion()).toEqual('1.0');

		expect(msg.setHttpVersion('1')).toBe(msg);
	});

	//TODO: dopisac test dla ciastek
});
