describe('Response', function(){
	var Response = null;

	it('deps', function(){
		expect(function(){
			Response = require('../../../lib/core/net/Response.js').Response;
		}).not.toThrow();
	});

	it('set/get statusCode', function(){
		var resp;

		expect(function(){ resp = new Response(); }).not.toThrow();

		expect(function(){ resp.setStatusCode(); }).toThrow(Response.Exception.WRONG_STATUS_CODE);
		expect(function(){ resp.setStatusCode(null); }).toThrow(Response.Exception.WRONG_STATUS_CODE);
		expect(function(){ resp.setStatusCode(undefined); }).toThrow(Response.Exception.WRONG_STATUS_CODE);
		expect(function(){ resp.setStatusCode(123); }).not.toThrow(Response.Exception.WRONG_STATUS_CODE);
		expect(function(){ resp.setStatusCode("123"); }).toThrow(Response.Exception.WRONG_STATUS_CODE);
		expect(function(){ resp.setStatusCode([]); }).toThrow(Response.Exception.WRONG_STATUS_CODE);
		expect(function(){ resp.setStatusCode({}); }).toThrow(Response.Exception.WRONG_STATUS_CODE);
		expect(function(){ resp.setStatusCode(function(){}); }).toThrow(Response.Exception.WRONG_STATUS_CODE);
		expect(function(){ resp.setStatusCode(new (function(){})); }).toThrow(Response.Exception.WRONG_STATUS_CODE);

		expect(resp.setStatusCode(200)).toBe(resp);
		expect(resp.getStatusCode()).toEqual(200);
	});

	it('isContentType', function(){
		var resp;

		expect(function(){ resp = new Response(); }).not.toThrow();

		expect(function(){ resp.isContentType(); }).toThrow(Response.Exception.WRONG_CONTENT_TYPE);
		expect(function(){ resp.isContentType(null); }).toThrow(Response.Exception.WRONG_CONTENT_TYPE);
		expect(function(){ resp.isContentType(undefined); }).toThrow(Response.Exception.WRONG_CONTENT_TYPE);
		expect(function(){ resp.isContentType(123); }).toThrow(Response.Exception.WRONG_CONTENT_TYPE);
		expect(function(){ resp.isContentType("123"); }).not.toThrow(Response.Exception.WRONG_CONTENT_TYPE);
		expect(function(){ resp.isContentType([]); }).toThrow(Response.Exception.WRONG_CONTENT_TYPE);
		expect(function(){ resp.isContentType({}); }).toThrow(Response.Exception.WRONG_CONTENT_TYPE);
		expect(function(){ resp.isContentType(function(){}); }).toThrow(Response.Exception.WRONG_CONTENT_TYPE);
		expect(function(){ resp.isContentType(new (function(){})); }).toThrow(Response.Exception.WRONG_CONTENT_TYPE);
	
		expect(function(){ resp.setHeader('content-type', 'application/json'); }).not.toThrow();

		expect(resp.isContentType('*/*')).toBeTruthy();
		expect(resp.isContentType('application/*')).toBeTruthy();
		expect(resp.isContentType('text/*')).toBeFalsy();
		expect(resp.isContentType('*/json')).toBeTruthy();
		expect(resp.isContentType('*/html')).toBeFalsy();
		expect(resp.isContentType('application/json')).toBeTruthy();
		expect(resp.isContentType('text/html')).toBeFalsy();
	});

	it('setBody', function(){
		var resp;

		expect(function(){ resp = new Response(); }).not.toThrow();

		expect(function(){ resp.setBody(); }).toThrow(Response.Exception.WRONG_BODY);
		expect(function(){ resp.setBody(null); }).toThrow(Response.Exception.WRONG_BODY);
		expect(function(){ resp.setBody(undefined); }).toThrow(Response.Exception.WRONG_BODY);
		expect(function(){ resp.setBody(123); }).toThrow(Response.Exception.WRONG_BODY);
		expect(function(){ resp.setBody("123"); }).not.toThrow(Response.Exception.WRONG_BODY);
		expect(function(){ resp.setBody([]); }).toThrow(Response.Exception.WRONG_BODY);
		expect(function(){ resp.setBody({}); }).toThrow(Response.Exception.WRONG_BODY);
		expect(function(){ resp.setBody(function(){}); }).toThrow(Response.Exception.WRONG_BODY);
		expect(function(){ resp.setBody(new (function(){})); }).toThrow(Response.Exception.WRONG_BODY);

		expect(resp.setBody("testowe cialko")).toBe(resp);
		expect(resp.getBody()).toEqual("testowe cialko");
	});

	it('toJson', function(){
		var resp = new Response();

		resp.setHeaders({'host': 'abc.pl'}).setStatusCode(200).setHttpVersion('1.1').setBody('qwe123');
		expect(resp.toJson()).toEqual({
			'httpVersion': '1.1',
			'statusCode': 200,
			'headers': {
				'host': 'abc.pl'
			},
			'body': 'qwe123'
		});
	});
});
