describe('AbsoluteLinkDecorator', function(){

    var AbsoluteLinkDecorator = null;
    var AbstractDecorator = null;
    var Request = null;
    var Response = null;
    var ErrorEvent = null;
    var Event = null;
    var ald = null;

    it('can load needed classes', function(){
        expect(function(){
            AbsoluteLinkDecorator = require('../../lib/decorator/AbsoluteLinkDecorator.js').AbsoluteLinkDecorator;
        }).not.toThrow();

        expect(function(){
            AbstractDecorator = require("../../lib/core/decorator/AbstractDecorator.js").AbstractDecorator;
        }).not.toThrow();

        expect(function(){
            Request = require("../../lib/core/net/Request.js").Request;
        }).not.toThrow();

        expect(function(){
            Response = require("../../lib/core/net/Response.js").Response;
        }).not.toThrow();

        expect(function(){
            Event = require('../../lib/core/event/Event.js').Event;
        }).not.toThrow();

        expect(function(){
            ErrorEvent = require('../../lib/core/event/ErrorEvent.js').ErrorEvent;
        }).not.toThrow();
    });
    

    it('proper init/construct', function(){
        expect(function(){
            ald = new AbsoluteLinkDecorator();
        }).not.toThrow();
    });
    

    it('checking invalid request object', function(){
        var responseEvent = null;

        runs(function(){
            ald.addEventListener(AbstractDecorator.ERROR, function(e){responseEvent = e;});
            ald.decorateRequest({});
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).toBeTruthy();
        });

    });

    it('checking empty request object', function(){
        var responseEvent = null;
        var request = new Request();
        ald = new AbsoluteLinkDecorator();


        runs(function(){
            ald.addEventListener(AbstractDecorator.COMPLETE, function(e){responseEvent = e;});
            ald.decorateRequest(request);
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).not.toBeTruthy();
        });

    });
    
    it('checking invalid response object', function(){
        var responseEvent = null;
        ald = new AbsoluteLinkDecorator();

        runs(function(){
            ald.addEventListener(AbstractDecorator.ERROR, function(e){responseEvent = e;});
            ald.decorateResponse({});
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).toBeTruthy();
        });

    });

    it('checking empty response object with no prior request', function(){
        var responseEvent = null;
        var response = new Response();
        ald = new AbsoluteLinkDecorator();


        runs(function(){
            ald.addEventListener(AbstractDecorator.COMPLETE, function(e){responseEvent = e;});
            ald.addEventListener(AbstractDecorator.ERROR, function(e){responseEvent = e;});
            ald.decorateResponse(response);
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).toBeTruthy();
        });

    });



    it('checking proper decoration', function(){
        
        var responseEvent = null;
        var request = null;
        var response = null;

        var testDecoration = function(host, body){

            responseEvent = null;
            
            request = new Request();
            request.setHeader('host', host);

            response = new Response();
            response.setBody(body);

            ald = new AbsoluteLinkDecorator();

            runs(function(){
                ald.addEventListener(AbstractDecorator.COMPLETE, function(eRequest){
                    ald.removeAllEventListeners(AbstractDecorator.COMPLETE);
                    ald.addEventListener(AbstractDecorator.COMPLETE, function(eResponse){
                        responseEvent = eResponse;
                    });
                    ald.decorateResponse(response);
                });
                ald.addEventListener(AbstractDecorator.ERROR, function(e){responseEvent = e;});
                ald.decorateRequest(request);
            });

        };

        var host = null;
        var body = null;
        var after = null;


        // test case 01
        runs(function(){
            host = 'onet.pl';
            body = '<a href="http://www.onet.pl/">LINK</a>';
            after = '<a href="/">LINK</a>';
            
            runs(function(){
                testDecoration(host, body);
            });
            waitsFor(function(){
                return responseEvent !== null;
            }, 1000);
            runs(function(){
               expect(response.getBody()).toBe(after);
            });
        });

        // test case 02
        runs(function(){
            host = 'onet.pl';
            body = '<a href="http://www.onet.pl/test.html">LINK</a>';
            after = '<a href="/test.html">LINK</a>';

            runs(function(){
                testDecoration(host, body);
            });
            waitsFor(function(){
                return responseEvent !== null;
            }, 1000);
            runs(function(){
               expect(response.getBody()).toBe(after);
            });
        });

        // test case 03
        runs(function(){
            host = 'onet.pl';
            body = '<a href="http://www.onet.pl/super/test.html">LINK</a>';
            after = '<a href="/super/test.html">LINK</a>';

            runs(function(){
                testDecoration(host, body);
            });
            waitsFor(function(){
                return responseEvent !== null;
            }, 1000);
            runs(function(){
               expect(response.getBody()).toBe(after);
            });
        });


    });


});