describe('CacheControlDecorator', function(){

    var CacheControlDecorator = null;
    var AbstractDecorator = null;
    var Request = null;
    var Response = null;
    var ErrorEvent = null;
    var Event = null;
    var ccd = null;

    it('can load needed classes', function(){
        expect(function(){
            CacheControlDecorator = require('../../lib/decorator/CacheControlDecorator.js').CacheControlDecorator;
        }).not.toThrow();

        expect(function(){
            AbstractDecorator = require("../../lib/core/decorator/AbstractDecorator.js").AbstractDecorator;
        }).not.toThrow();

        expect(function(){
            Request = require("../../lib/core/net/Request.js").Request;
        }).not.toThrow();

        expect(function(){
            Response = require("../../lib/core/net/Response.js").Response;
        }).not.toThrow();

        expect(function(){
            Event = require('../../lib/core/event/Event.js').Event;
        }).not.toThrow();

        expect(function(){
            ErrorEvent = require('../../lib/core/event/ErrorEvent.js').ErrorEvent;
        }).not.toThrow();
    });

    it('proper init/construct', function(){
        expect(function(){
            ccd = new CacheControlDecorator();
        }).not.toThrow();
    });

    it('decorate request is not used', function(){
        var responseEvent = null;

        runs(function(){
            ccd.addEventListener(AbstractDecorator.COMPLETE, function(e){responseEvent = e;});
            ccd.decorateRequest({});
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).toBeFalsy();
        });

    });


    it('checking invalid response object', function(){
        var responseEvent = null;

        runs(function(){
            ccd.addEventListener(AbstractDecorator.ERROR, function(e){responseEvent = e;});
            ccd.decorateResponse({});
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).toBeTruthy();
        });

    });


    it('checking proper decoration', function(){

        var responseEvent = null;
        var request = null;
        var response = null;

        var testDecoration = function(setCacheControlHeader){
            
            responseEvent = null;
            
            request = new Request();
            response = new Response();
            response.setHeader('cache-control', setCacheControlHeader);
            
            ccd = new CacheControlDecorator();

            runs(function(){
                ccd.addEventListener(AbstractDecorator.COMPLETE, function(eRequest){
                    ccd.removeAllEventListeners(AbstractDecorator.COMPLETE);
                    ccd.addEventListener(AbstractDecorator.COMPLETE, function(eResponse){
                        responseEvent = eResponse;
                    });
                    ccd.decorateResponse(response);
                });
                ccd.addEventListener(AbstractDecorator.ERROR, function(e){responseEvent = e;});
                ccd.decorateRequest(request);
            });

        };

        var requestCacheControl = null;
        var decoratedCacheControl = null;

        // test case 01
        runs(function(){
            requestCacheControl = 'strange';
            decoratedCacheControl = 'server';

            runs(function(){
                testDecoration(requestCacheControl);
            });
            waitsFor(function(){
                return responseEvent !== null;
            }, 1000);
            runs(function(){
               expect(response.getHeader('cache-control')).toBe(decoratedCacheControl);
            });
        });

        // test case 02
        runs(function(){
            requestCacheControl = undefined;
            decoratedCacheControl = 'server';

            runs(function(){
                testDecoration(requestCacheControl);
            });
            waitsFor(function(){
                return responseEvent !== null;
            }, 1000);
            runs(function(){
               expect(response.getHeader('cache-control')).toBe(decoratedCacheControl);
            });
        });

        // test case 03
        runs(function(){
            requestCacheControl = 'server';
            decoratedCacheControl = 'server';

            runs(function(){
                testDecoration(requestCacheControl);
            });
            waitsFor(function(){
                return responseEvent !== null;
            }, 1000);
            runs(function(){
               expect(response.getHeader('cache-control')).toBe(decoratedCacheControl);
            });
        });

        // test case 04
        runs(function(){
            requestCacheControl = 'no-cache';
            decoratedCacheControl = 'no-cache';

            runs(function(){
                testDecoration(requestCacheControl);
            });
            waitsFor(function(){
                return responseEvent !== null;
            }, 1000);
            runs(function(){
               expect(response.getHeader('cache-control')).toBe(decoratedCacheControl);
            });
        });

        // test case 0X
        // TODO kubek ma zmienic implementacje setHeader bo nei dziala z nullem
        /*
        runs(function(){
            requestCacheControl = null;
            decoratedCacheControl = 'server';

            runs(function(){
                testDecoration(requestCacheControl);
            });
            waitsFor(function(){
                return responseEvent !== null;
            }, 1000);
            runs(function(){
               expect(response.getHeader('cache-control')).toBe(decoratedCacheControl);
            });
        });
        */



    });

});