describe('CookieDecorator', function(){


    var CookieDecorator = null;
    var AbstractDecorator = null;
    var Request = null;
    var Response = null;
    var ErrorEvent = null;
    var Event = null;
    var cd = null;

    it('can load needed classes', function(){
        expect(function(){
            CookieDecorator = require('../../lib/decorator/CookieDecorator.js').CookieDecorator;
        }).not.toThrow();

        expect(function(){
            AbstractDecorator = require("../../lib/core/decorator/AbstractDecorator.js").AbstractDecorator;
        }).not.toThrow();

        expect(function(){
            Request = require("../../lib/core/net/Request.js").Request;
        }).not.toThrow();

        expect(function(){
            Response = require("../../lib/core/net/Response.js").Response;
        }).not.toThrow();

        expect(function(){
            Event = require('../../lib/core/event/Event.js').Event;
        }).not.toThrow();

        expect(function(){
            ErrorEvent = require('../../lib/core/event/ErrorEvent.js').ErrorEvent;
        }).not.toThrow();
    });

    it('proper init/construct', function(){
        expect(function(){
            cd = new CookieDecorator();
        }).not.toThrow();
    });
    

    it('checking invalid request object', function(){
        var responseEvent = null;

        runs(function(){
            cd.addEventListener(AbstractDecorator.ERROR, function(e){responseEvent = e;});
            cd.decorateRequest({});
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).toBeTruthy();
        });

    });

    it('checking empty request object', function(){
        var responseEvent = null;
        var request = new Request();
        cd = new CookieDecorator();


        runs(function(){
            cd.addEventListener(AbstractDecorator.COMPLETE, function(e){responseEvent = e;});
            cd.decorateRequest(request);
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).not.toBeTruthy();
        });

    });

    it('checking invalid response object', function(){
        var responseEvent = null;
        cd = new CookieDecorator();

        runs(function(){
            cd.addEventListener(AbstractDecorator.ERROR, function(e){responseEvent = e;});
            cd.decorateResponse({});
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).toBeTruthy();
        });

    });

    it('decorating request - forwarding onet cookies', function(){
        var responseEvent = null;
        cd = new CookieDecorator();
        cd.configuration.forwardOnetCookie = true;
        var request = new Request();
        request.setHeader('cookie', 'onet_ubi=123;foo=bar;');

        runs(function(){
            cd.addEventListener(AbstractDecorator.COMPLETE, function(e){responseEvent = e;});
            cd.decorateRequest(request);
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
            expect(request.getHeader('cookie')).toBe('onet_ubi=123;foo=bar;');
        });

    });

    xit('decorating request - not forwarding onet cookies', function(){
        var responseEvent = null;
        cd = new CookieDecorator();
        var request = new Request();
        request.setHeader('cookie', 'onet_ubi=123;foo=bar;');

        runs(function(){
            cd.addEventListener(AbstractDecorator.COMPLETE, function(e){responseEvent = e;});
            cd.decorateRequest(request);
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
            expect(request.getHeader('cookie')).toBe('foo=bar;');
        });

    });


    it('decorating response - deleting set cookies', function(){
        var responseEvent = null;
        cd = new CookieDecorator();
        cd.configuration.disallowSet = true;
        var response = new Response();
        response.setHeader('set-cookie', 'something');

        runs(function(){
            cd.addEventListener(AbstractDecorator.COMPLETE, function(e){responseEvent = e;});
            cd.decorateResponse(response);
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(response.getHeader('set-cookie')).toBeNull();
        });

    });

    it('decorating response - not deleting set cookies', function(){
        var responseEvent = null;
        cd = new CookieDecorator();
        var response = new Response();
        response.setHeader('set-cookie', 'something');

        runs(function(){
            cd.addEventListener(AbstractDecorator.COMPLETE, function(e){responseEvent = e;});
            cd.decorateResponse(response);
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
            expect(response.getHeader('set-cookie')).not.toBeNull();
        });

    });

});