describe('PathDecorator', function(){

    var PathDecorator = null;
    var AbstractDecorator = null;
    var Request = null;
    var Response = null;
    var ErrorEvent = null;
    var Event = null;
    var pd = null;

    it('can load needed classes', function(){
        expect(function(){
            PathDecorator = require('../../lib/decorator/PathDecorator.js').PathDecorator;
        }).not.toThrow();

        expect(function(){
            AbstractDecorator = require("../../lib/core/decorator/AbstractDecorator.js").AbstractDecorator;
        }).not.toThrow();

        expect(function(){
            Request = require("../../lib/core/net/Request.js").Request;
        }).not.toThrow();

        expect(function(){
            Response = require("../../lib/core/net/Response.js").Response;
        }).not.toThrow();

        expect(function(){
            Event = require('../../lib/core/event/Event.js').Event;
        }).not.toThrow();

        expect(function(){
            ErrorEvent = require('../../lib/core/event/ErrorEvent.js').ErrorEvent;
        }).not.toThrow();
    });


    it('proper init/construct', function(){
        expect(function(){
            pd = new PathDecorator();
        }).not.toThrow();
    });


    it('checking invalid request object', function(){
        var responseEvent = null;

        runs(function(){
            pd.addEventListener(AbstractDecorator.ERROR, function(e){responseEvent = e;});
            pd.decorateRequest({});
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).toBeTruthy();
        });

    });

    it('checking empty request object', function(){
        var responseEvent = null;
        var request = new Request();
        pd = new PathDecorator();


        runs(function(){
            pd.addEventListener(AbstractDecorator.COMPLETE, function(e){responseEvent = e;});
            pd.decorateRequest(request);
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).not.toBeTruthy();
        });

    });

    it('decorate request is not used', function(){
        var responseEvent = null;
        pd = new PathDecorator();

        runs(function(){
            pd.addEventListener(AbstractDecorator.COMPLETE, function(e){responseEvent = e;});
            pd.decorateResponse({});
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).toBeFalsy();
        });

    });

    it('decorating request - changing path', function(){
        var responseEvent = null;

        var request = new Request();
        pd = new PathDecorator();

        pd.configuration = {
            'source' : 'www.onet.pl',
            'destination': 'proxy.onet.pl'
        };
        request.setPath('http://www.onet.pl/index.html');

        runs(function(){
            pd.addEventListener(AbstractDecorator.COMPLETE, function(e){responseEvent = e;});
            pd.decorateRequest(request);
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
            expect(request.getPath()).toBe('http://proxy.onet.pl/index.html');
        });

    });

    it('decorating request - changing path', function(){
        var responseEvent = null;

        var request = new Request();
        pd = new PathDecorator();

        pd.configuration = {
            'source' : 'www.onet.pl',
            'destination': 'proxy.onet.pl'
        };
        request.setPath('http://www.onet.pl/news/proxy.onet.pl/');

        runs(function(){
            pd.addEventListener(AbstractDecorator.COMPLETE, function(e){responseEvent = e;});
            pd.decorateRequest(request);
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
            expect(request.getPath()).toBe('http://proxy.onet.pl/news/proxy.onet.pl/');
        });

    });

});