describe('RevProxyDecorator', function(){


    var RevProxyDecorator = null;
    var AbstractDecorator = null;
    var Request = null;
    var Response = null;
    var ErrorEvent = null;
    var Event = null;
    var rpd = null;

    it('can load needed classes', function(){
        expect(function(){
            RevProxyDecorator = require('../../lib/decorator/RevProxyDecorator.js').RevProxyDecorator;
        }).not.toThrow();

        expect(function(){
            AbstractDecorator = require("../../lib/core/decorator/AbstractDecorator.js").AbstractDecorator;
        }).not.toThrow();

        expect(function(){
            Request = require("../../lib/core/net/Request.js").Request;
        }).not.toThrow();

        expect(function(){
            Response = require("../../lib/core/net/Response.js").Response;
        }).not.toThrow();

        expect(function(){
            Event = require('../../lib/core/event/Event.js').Event;
        }).not.toThrow();

        expect(function(){
            ErrorEvent = require('../../lib/core/event/ErrorEvent.js').ErrorEvent;
        }).not.toThrow();
    });

    it('proper init/construct', function(){
        expect(function(){
            rpd = new RevProxyDecorator();
        }).not.toThrow();
    });

    it('checking invalid request object', function(){
        var responseEvent = null;
        rpd = new RevProxyDecorator();

        runs(function(){
            rpd.addEventListener(AbstractDecorator.ERROR, function(e){responseEvent = e;});
            rpd.decorateResponse({});
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).toBeTruthy();
        });

    });


    it('checking empty request object', function(){
        var responseEvent = null;
        var request = new Request();
        rpd = new RevProxyDecorator();


        runs(function(){
            rpd.addEventListener(AbstractDecorator.COMPLETE, function(e){responseEvent = e;});
            rpd.decorateRequest(request);
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).not.toBeTruthy();
        });

    });

    it('decorate request is not used', function(){
        var responseEvent = null;

        runs(function(){
            rpd.addEventListener(AbstractDecorator.COMPLETE, function(e){responseEvent = e;});
            rpd.decorateRequest({});
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).toBeFalsy();
        });

    });


    it('decorate response case 01', function(){
        var responseEvent = null;
        var response = new Response();
        rpd = new RevProxyDecorator();
        
        runs(function(){
            rpd.addEventListener(AbstractDecorator.COMPLETE, function(e){responseEvent = e;});
            rpd.decorateResponse(response);
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).toBeFalsy();
        });
    });

    it('decorate response case 02', function(){
        var responseEvent = null;
        var response = new Response();
        response.setBody('<html><title>test</title></html>');
        var output = '<html><cfd:model content-type="text/html" target="title">\r\n<title>test</title>\r\n</cdf:model></html>';

        rpd = new RevProxyDecorator();
        rpd.configuration = {
            "section": [
                {
                    "name": "title",
                    "source": "document:title"
                }
            ]
        };

        runs(function(){
            rpd.addEventListener(AbstractDecorator.COMPLETE, function(e){responseEvent = e;});
            rpd.decorateResponse(response);
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).toBeFalsy();
           expect(response.getBody()).toEqual(output)
        });
    });

    it('decorate response case 03', function(){
        var responseEvent = null;
        var response = new Response();
        response.setBody('<html><title>test</title><body>Hello</body></html>');
        var output = '<html><title>test</title><cfd:model content-type="text/html" target="body">\r\n<body>Hello</body>\r\n</cdf:model></html>';

        rpd = new RevProxyDecorator();
        rpd.configuration = {
            "section": [
                {
                    "name": "body",
                    "source": "document:body"
                }
            ]
        };

        runs(function(){
            rpd.addEventListener(AbstractDecorator.COMPLETE, function(e){responseEvent = e;});
            rpd.decorateResponse(response);
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).toBeFalsy();
           expect(response.getBody()).toEqual(output)
        });
    });

    it('decorate response case 04 invalid section name', function(){
        var responseEvent = null;
        var response = new Response();
        response.setBody('<html><title>test</title></html>');
        var output = '<html><cfd:model content-type="text/html" target="title">\r\n<title>test</title>\r\n</cdf:model></html>';

        rpd = new RevProxyDecorator();
        rpd.configuration = {
            "section": [
                {
                    "source": "document:title"
                }
            ]
        };

        runs(function(){
            rpd.addEventListener(AbstractDecorator.ERROR, function(e){responseEvent = e;});
            rpd.decorateResponse(response);
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).toBeTruthy();
           expect(responseEvent.data).toEqual('There is no \'name\' field which is required!');
        });
    });

    it('decorate response case 05 invalid pattern', function(){
        var responseEvent = null;
        var response = new Response();
        response.setBody('<html><title>test</title></html>');

        rpd = new RevProxyDecorator();
        rpd.configuration = {
            "section": [
                {
                    "name"  : "title",
                    "source": "document:title2"
                }
            ]
        };

        runs(function(){
            rpd.addEventListener(AbstractDecorator.ERROR, function(e){responseEvent = e;});
            rpd.decorateResponse(response);
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).toBeTruthy();
           expect(responseEvent.data).toEqual('There is no pattern for section named: \'title\' in RevProxyDecorator class.');
        });
    });

    it('decorate response case 06 prepend', function(){
        var responseEvent = null;
        var response = new Response();
        response.setBody('<html><title>test</title><body>Hello</body></html>');
        var output = '<html><title>test</title><body>Hello</body></html>';

        rpd = new RevProxyDecorator();
        rpd.configuration = {
            "section": [
                {
                    "name": "body",
                    "begin": "onet_not_exists_begin",
                    "end": "onet_not_exists_end"
                }
            ]
        };

        runs(function(){
            rpd.addEventListener(AbstractDecorator.COMPLETE, function(e){responseEvent = e;});
            rpd.decorateResponse(response);
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).toBeFalsy();
           expect(response.getBody()).toEqual(output)
        });
    });

    it('decorate response case 06 prepend and append', function(){
        var responseEvent = null;
        var response = new Response();
        response.setBody('<html><title>test</title><body><!-- onet_section_begin -->Hello<!-- onet_section_end --></body></html>');
        var output = '<html><title>test</title><body><cfd:model content-type="text/html" target="example">\r\n<b>prepend</b>Hello<b>append</b>\r\n</cdf:model></body></html>';

        rpd = new RevProxyDecorator();
        rpd.configuration = {
            "section": [
                {
                    "name": "example",
                    "begin": "onet_section_begin",
                    "end": "onet_section_end",
                    "prepend" : "<b>prepend</b>",
                    "append"  : "<b>append</b>"
                }
            ]
        };

        runs(function(){
            rpd.addEventListener(AbstractDecorator.COMPLETE, function(e){responseEvent = e;});
            rpd.decorateResponse(response);
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).toBeFalsy();
           expect(response.getBody()).toEqual(output)
        });
    });

    it('decorate response case 06 subst', function(){
        var responseEvent = null;
        var response = new Response();
        response.setBody('<html><title>test</title><body><!-- onet_section_begin --><!-- onet_section_end --></body></html>');
        var output = '<html><title>test</title><body><cfd:model content-type="text/html" target="example">\r\nsomething else!\r\n</cdf:model></body></html>';

        rpd = new RevProxyDecorator();
        rpd.configuration = {
            "section": [
                {
                    "name": "example",
                    "begin": "onet_section_begin",
                    "end": "onet_section_end",
                    "subst" : "something else!"
                }
            ]
        };

        runs(function(){
            rpd.addEventListener(AbstractDecorator.COMPLETE, function(e){responseEvent = e;});
            rpd.decorateResponse(response);
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).toBeFalsy();
           expect(response.getBody()).toEqual(output)
        });
    });

    it('decorate response case 07 subst regexp test with whitespace trim', function(){
        var responseEvent = null;
        var response = new Response();
        response.setBody('<html><title>test</title><body><!-- onet_section_begin --> \r\n  <!-- onet_section_end --></body></html>');
        var output = '<html><title>test</title><body><cfd:model content-type="text/html" target="example">\r\nsomething else!\r\n</cdf:model></body></html>';

        rpd = new RevProxyDecorator();
        rpd.configuration = {
            "section": [
                {
                    "name": "example",
                    "begin": "onet_section_begin",
                    "end": "onet_section_end",
                    "subst" : "something else!"
                }
            ]
        };

        runs(function(){
            rpd.addEventListener(AbstractDecorator.COMPLETE, function(e){responseEvent = e;});
            rpd.decorateResponse(response);
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).toBeFalsy();
           expect(response.getBody()).toEqual(output)
        });
    });


    it('decorate response case 08 null pattern', function(){
        var responseEvent = null;
        var response = new Response();
        response.setBody('<html><title>test</title><body><!-- onet_section_begin --> \r\n  <!-- onet_section_end --></body></html>');

        rpd = new RevProxyDecorator();
        rpd.configuration = {
            "section": [
                {
                    "name": "example",
                }
            ]
        };

        runs(function(){
            rpd.addEventListener(AbstractDecorator.ERROR, function(e){responseEvent = e;});
            rpd.decorateResponse(response);
        });

        waitsFor(function(){
            return responseEvent !== null;
        }, 1000);

        runs(function(){
           expect(responseEvent instanceof ErrorEvent).toBeTruthy();
           expect(responseEvent.data).toEqual("There is no pattern for section named: 'example' in RevProxyDecorator class.");
        });
    });

    

});